@@ -19,7 +19,7 @@ running_as_root() {
 
 VERBOSE=""
 ASKCONFIRMATION=1
-OVERWRITEDIR="ask"
+RECREATEDEST="ask"
 USEVIRTUALENV="yes"
 
 # ansi colors for formatting heredoc
@@ -28,13 +28,13 @@ GREEN="$ESC[0;32m"
 NO_COLOR="$ESC[0;0m"
 RED="$ESC[0;31m"
 
-PYTHON2="$(which python || true)"
-PYTHON3="$(which python3 || true)"
+PYTHON2=$(which python || true)
+PYTHON3=$(which python3 || true)
 PYTHON=${PYTHON2:-$PYTHON3}
 BASE_PIP_URL=${BASE_PIP_URL:-"https://pypi.python.org/simple"}
 VIRTUALENV_191_URL="https://raw.github.com/pypa/virtualenv/1.9.1/virtualenv.py"
 
-RALLY_GIT_URL="https://github.com/openstack/rally"
+RALLY_GIT_URL="https://git.openstack.org/openstack/rally"
 RALLY_GIT_BRANCH="master"
 RALLY_CONFIGURATION_DIR=/etc/rally
 RALLY_DATABASE_DIR=/var/lib/rally/database
@@ -164,12 +164,16 @@ $GREEN  -d, --target DIRECTORY$NO_COLOR Install Rally virtual environment into D
 $GREEN  --url                 $NO_COLOR Git repository public URL to download Rally from.
                          This is useful when you have only installation script and want to install Rally
                          from custom repository.
-                         (Default: https://github.com/openstack/rally).
+                         (Default: ${RALLY_GIT_URL}).
                          (Ignored when you are already in git repository).
 $GREEN  --branch              $NO_COLOR Git branch name name or git tag (Rally release) to install.
                          (Default: latest - master).
                          (Ignored when you are already in git repository).
-$GREEN  -f, --overwrite       $NO_COLOR Remove target directory if it already exist.
+$GREEN  -f, --overwrite       $NO_COLOR Deprecated. Use -r instead.
+$GREEN  -r, --recreate        $NO_COLOR Remove target directory if it already exist.
+                         If neither '-r' nor '-R' is set default behaviour is to ask.
+$GREEN  -R, --no-recreate     $NO_COLOR Do not reemove target directory if it already exist.
+                         If neither '-r' nor '-R' is set default behaviour is to ask.
 $GREEN  -y, --yes             $NO_COLOR Do not ask for confirmation: assume a 'yes' reply
                          to every question.
 $GREEN  -D, --dbtype TYPE     $NO_COLOR Select the database type. TYPE can be one of
@@ -295,7 +299,7 @@ install_required_sw () {
         fi
     elif have_command yum; then
         # RHEL/CentOS
-        missing=$(which_missing_packages gcc libffi-devel python-devel openssl-devel gmp-devel libxml2-devel libxslt-devel postgresql-devel git wget)
+        missing=$(which_missing_packages gcc libffi-devel python-devel openssl-devel gmp-devel libxml2-devel libxslt-devel postgresql-devel redhat-rpm-config git wget)
 
         if [ "$ASKCONFIRMATION" -eq 0 ]; then
             pkg_manager="yum install -y"
@@ -313,7 +317,7 @@ install_required_sw () {
         fi
     else
         # MacOSX maybe?
-        echo "Cannot determine what package manager this Linux distribution has, so I cannot check if requisite software is installed. I'm proceeding anyway, but you may run into errors later."
+        echo "Cannot determine what package manager this system has, so I cannot check if requisite software is installed. I'm proceeding anyway, but you may run into errors later."
     fi
     if ! have_command pip; then
         missing="$missing python-pip"
@@ -455,14 +459,28 @@ setup_rally_configuration () {
     cp "$SRCDIR"/etc/rally/rally.conf.sample "$ETCDIR"/rally.conf
 
     [ -d "$DBDIR" ] || mkdir -p "$DBDIR"
-    sed -i "s|#connection *=.*|connection = \"$DBCONNSTRING\"|" "$ETCDIR"/rally.conf
+    local CONF_TMPFILE=$(mktemp)
+    sed "s|#connection *=.*|connection = \"$DBCONNSTRING\"|" "$ETCDIR"/rally.conf > "$CONF_TMPFILE"
+    cat "$CONF_TMPFILE" > "$ETCDIR"/rally.conf
+    rm "$CONF_TMPFILE"
     rally-manage db recreate
 }
 
+rally_venv () {
+    echo "Installing Rally virtualenv in directory '$VENVDIR' ..."
+    CURRENT_ACTION="creating-venv"
+    if ! install_virtualenv "$VENVDIR"; then
+        die $EX_PROTOCOL "Unable to create a new virtualenv in '$VENVDIR': 'virtualenv.py' script exited with code $rc." <<__EOF__
+The script was unable to create a valid virtual environment.
+__EOF__
+    fi
+    CURRENT_ACTION="venv-created"
+    rc=0
+}
 
 ### Main program ###
-short_opts='d:vsyfhD:p:'
-long_opts='target:,verbose,overwrite,system,yes,dbtype:,python:,db-user:,db-password:,db-host:,db-name:,help,url:,branch:,develop,no-color'
+short_opts='d:vsyfrRhD:p:'
+long_opts='target:,verbose,overwrite,recreate,no-recreate,system,yes,dbtype:,python:,db-user:,db-password:,db-host:,db-name:,help,url:,branch:,develop,no-color'
 
 set +e
 if [ "x$(getopt -T)" = 'x' ]; then
@@ -502,7 +520,13 @@ do
             USEVIRTUALENV="no"
             ;;
         -f|--overwrite)
-            OVERWRITEDIR=yes
+            RECREATEDEST=yes
+            ;;
+        -r|--recreate)
+            RECREATEDEST=yes
+            ;;
+        -R|--no-recreate)
+            RECREATEDEST=no
             ;;
         -y|--yes)
             ASKCONFIRMATION=0
@@ -619,18 +643,23 @@ $GREEN
 __EOF__
     fi
     DBAUTH="$DBUSER:$DBPASSWORD@$DBHOST"
-    DBCONNSTRING="$DBTYPE://$DBAUTH/$DBNAME"
+    if [ "$DBTYPE" = 'mysql' ]; then
+        DBCONNSTRING="$DBTYPE+pymysql://$DBAUTH/$DBNAME"
+    else
+        DBCONNSTRING="$DBTYPE://$DBAUTH/$DBNAME"
+    fi
 fi
 
 # check and install prerequisites
 install_required_sw
 require_python
 
+
 # Install virtualenv, if required
 if [ "$USEVIRTUALENV" = 'yes' ]; then
     if [ -d "$VENVDIR" ]
     then
-        if [ $OVERWRITEDIR = 'ask' ]; then
+        if [ $RECREATEDEST = 'ask' ]; then
             echo "Destination directory '$VENVDIR' already exists."
             echo "I can wipe it out in order to make a new installation,"
             echo "but this means any files in that directory, and the ones"
@@ -640,44 +669,28 @@ if [ "$USEVIRTUALENV" = 'yes' ]; then
             if ! ask_yn "Do you want to wipe the installation directory '$VENVDIR'?"
             then
                 echo "*Not* overwriting destination directory '$VENVDIR'."
-                OVERWRITEDIR=no
+                RECREATEDEST=no
             else
-                echo "Removing directory $VENVDIR as requested."
-                rm $VERBOSE -rf "$VENVDIR"
+                RECREATEDEST=yes
+
             fi
-        elif [ $OVERWRITEDIR = 'no' ]
+        fi
+
+        if [ $RECREATEDEST = 'yes' ];
         then
-            die $EX_CANTCREAT "Unable to create virtualenv in '$VENVDIR': directory already exists." <<__EOF__
-    The script was unable to create a virtual environment in "$VENVDIR"
-    because the directory already exists.
-
-    In order to proceed, you must take one of the following action:
-
-    * delete the directory, or
-
-    * run this script again adding '--overwrite' option, which will
-      overwrite the $VENVDIR directory, or
-
-    * specify a different path by running this script again adding the
-      option: "--target" followed by a non-existent directory.
-__EOF__
-        elif [ $OVERWRITEDIR = 'yes' ]; then
             echo "Removing directory $VENVDIR as requested."
             rm $VERBOSE -rf "$VENVDIR"
+            rally_venv
+        elif [ $RECREATEDEST = 'no' ];
+        then
+            echo "Using existing virtualenv at $VENVDIR..."
+            . "$VENVDIR"/bin/activate
         else
-            abort 66 "Internal error: unexpected value '$OVERWRITEDIR' for OVERWRITEDIR."
+            abort 66 "Internal error: unexpected value '$RECREATEDEST' for RECREATEDEST."
         fi
+    else
+        rally_venv
     fi
-
-    echo "Installing Rally virtualenv in directory '$VENVDIR' ..."
-    CURRENT_ACTION="creating-venv"
-    if ! install_virtualenv "$VENVDIR"; then
-        die $EX_PROTOCOL "Unable to create a new virtualenv in '$VENVDIR': 'virtualenv.py' script exited with code $rc." <<__EOF__
-The script was unable to create a valid virtual environment.
-__EOF__
-    fi
-    CURRENT_ACTION="venv-created"
-    rc=0
 fi
 
 # Install rally
@@ -697,19 +710,6 @@ then
         echo "Warning! Unable to wipe python compiled files"
     fi
 
-    if [ "$USEVIRTUALENV" = 'yes' ]
-    then
-        if [ "$VENVDIR/src" != "$BASEDIR" ]
-        then
-            SOURCEDIR="$VENVDIR"/src
-            if [ -d $SOURCEDIR ]
-            then
-                rm -rf $SOURCEDIR
-            fi
-            mkdir $SOURCEDIR
-            cp -r . $SOURCEDIR/
-        fi
-    fi
     popd > /dev/null
 else
     if [ "$USEVIRTUALENV" = 'yes' ]
@@ -719,29 +719,6 @@ else
         SOURCEDIR="$ORIG_WD"/rally.git
     fi
 
-    # Check if source directory is present
-    if [ -d "$SOURCEDIR" ]
-    then
-        if [ $OVERWRITEDIR != 'yes' ]
-        then
-            echo "Source directory '$SOURCEDIR' already exists."
-            echo "I can wipe it out in order to make a new installation,"
-            echo "but this means any files in that directory, and the ones"
-            echo "underneath it will be deleted."
-            echo
-            if ! ask_yn "Do you want to wipe the source directory '$SOURCEDIR'?"
-            then
-                echo "*Not* overwriting destination directory '$SOURCEDIR'."
-            else
-                rm -rf $SOURCEDIR
-                if [ -d "$SOURCEDIR"/.git ]
-                then
-                    abort $EX_CANTCREAT "Unable to wipe source directory $SOURCEDIR"
-                fi
-            fi
-        fi
-    fi
-
     if ! [ -d "$SOURCEDIR"/.git ]
     then
         echo "Downloading Rally from subversion repository $RALLY_GIT_URL ..."
@@ -793,16 +770,11 @@ __EOF__
         SAMPLESDIR=$VENVDIR/samples
         mkdir -p $SAMPLESDIR
         cp -r $SOURCEDIR/samples/* $SAMPLESDIR/
-        if [ "$BASEDR" != "$SOURCEDIR" ]
-        then
-            rm -rf $SOURCEDIR
-            echo "Source directory is removed."
-        else
-            echo "Unabled to remove source directory, becaus this script was started from it."
-        fi
     else
         SAMPLESDIR=$SOURCEDIR/samples
     fi
+    mkdir -p $VENVDIR/etc/bash_completion.d
+    install $SOURCEDIR/etc/rally.bash_completion $VENVDIR/etc/bash_completion.d/
 
     cat <<__EOF__
 $GREEN==============================
@@ -834,14 +806,6 @@ else
         SAMPLESDIR=/usr/share/rally/samples
         mkdir -p $SAMPLESDIR
         cp -r $SOURCEDIR/samples/* $SAMPLESDIR/
-        if [ "$BASEDIR" != "$SOURCEDIR" ]
-        then
-            rm -rf $SOURCEDIR
-            echo "Source directory is removed."
-        else
-            echo "Unabled to remove source directory, because this script was started from it."
-
-        fi
     else
         SAMPLESDIR=$SOURCEDIR/samples
     fi
