@@ -24,7 +24,7 @@ import jinja2.meta
 import jsonschema
 
 from rally.common.i18n import _, _LI
-from rally.common import log as logging
+from rally.common import logging
 from rally.common import objects
 from rally import consts
 from rally.deployment import engine as deploy_engine
@@ -65,8 +65,8 @@ class Deployment(object):
             raise
 
         with deployer:
-            endpoints = deployer.make_deploy()
-            deployment.update_endpoints(endpoints)
+            credentials = deployer.make_deploy()
+            deployment.update_credentials(credentials)
             return deployment
 
     @classmethod
@@ -99,8 +99,8 @@ class Deployment(object):
             deployment["config"]["type"], deployment)
         with deployer:
             deployer.make_cleanup()
-            endpoints = deployer.make_deploy()
-            deployment.update_endpoints(endpoints)
+            credentials = deployer.make_deploy()
+            deployment.update_credentials(credentials)
 
     @classmethod
     def get(cls, deployment):
@@ -119,12 +119,47 @@ class Deployment(object):
         :returns: Service list
         """
         # TODO(kun): put this work into objects.Deployment
-        clients = osclients.Clients(objects.Endpoint(**deployment["admin"]))
+        clients = osclients.Clients(objects.Credential(**deployment["admin"]))
         return clients.services()
 
+    @staticmethod
+    def list(status=None, parent_uuid=None, name=None):
+        """Get the deployments list.
+
+        :returns: Deployment list
+        """
+        return objects.Deployment.list(status, parent_uuid, name)
+
+    @classmethod
+    def check(cls, deployment):
+        """Check keystone authentication and list all available services.
+
+        :returns: Service list
+        """
+        services = cls.service_list(deployment)
+        users = deployment["users"]
+        for endpoint_dict in users:
+            osclients.Clients(objects.Credential(**endpoint_dict)).keystone()
+
+        return services
+
 
 class Task(object):
 
+    TASK_RESULT_SCHEMA = objects.task.TASK_RESULT_SCHEMA
+
+    @staticmethod
+    def list(**filters):
+        return objects.Task.list(**filters)
+
+    @staticmethod
+    def get(task_id):
+        return objects.Task.get(task_id)
+
+    @staticmethod
+    def get_detailed(task_id):
+        return objects.Task.get_detailed(task_id)
+
     @classmethod
     def render_template(cls, task_template, template_dir="./", **kwargs):
         """Render jinja2 task template to Rally input task.
@@ -156,13 +191,29 @@ class Task(object):
 
         env = jinja2.Environment(
             loader=jinja2.FileSystemLoader(template_dir, encoding="utf8"))
+        env.globals.update(cls.create_template_functions())
         ast = env.parse(task_template)
+        # NOTE(Julia Varigina):
+        # Bug in jinja2.meta.find_undeclared_variables
+        #
+        # The method shows inconsistent behavior:
+        # it does not return undeclared variables that appear
+        # in included templates only (via {%- include "some_template.yaml"-%})
+        # and in the same time is declared in jinja2.Environment.globals.
+        #
+        # This is different for undeclared variables that appear directly
+        # in task_template. The method jinja2.meta.find_undeclared_variables
+        # returns an undeclared variable that is used in task_template
+        # and is set in jinja2.Environment.globals.
+        #
+        # Despite this bug, jinja resolves values
+        # declared in jinja2.Environment.globals for both types of undeclared
+        # variables and successfully renders templates in both cases.
         required_kwargs = jinja2.meta.find_undeclared_variables(ast)
-
-        missing = set(required_kwargs) - set(kwargs) - set(dir(builtins))
+        missing = (set(required_kwargs) - set(kwargs) - set(dir(builtins)) -
+                   set(env.globals))
         real_missing = [mis for mis in missing
                         if is_really_missing(mis, task_template)]
-
         if real_missing:
             multi_msg = _("Please specify next template task arguments: %s")
             single_msg = _("Please specify template task argument: %s")
@@ -172,6 +223,25 @@ class Task(object):
 
         return env.from_string(task_template).render(**kwargs)
 
+    @classmethod
+    def create_template_functions(cls):
+
+        def template_min(int1, int2):
+            return min(int1, int2)
+
+        def template_max(int1, int2):
+            return max(int1, int2)
+
+        def template_round(float1):
+            return int(round(float1))
+
+        def template_ceil(float1):
+            import math
+            return int(math.ceil(float1))
+
+        return {"min": template_min, "max": template_max,
+                "ceil": template_ceil, "round": template_round}
+
     @classmethod
     def create(cls, deployment, tag):
         """Create a task without starting it.
@@ -197,7 +267,7 @@ class Task(object):
         deployment = objects.Deployment.get(deployment)
         task = task_instance or objects.Task(
             deployment_uuid=deployment["uuid"], temporary=True)
-        benchmark_engine = engine.BenchmarkEngine(
+        benchmark_engine = engine.TaskEngine(
             config, task, admin=deployment["admin"], users=deployment["users"])
 
         benchmark_engine.validate()
@@ -212,9 +282,8 @@ class Task(object):
         :param deployment: UUID or name of the deployment
         :param config: a dict with a task configuration
         :param task: Task object. If None, it will be created
-        :param abort_on_sla_failure: if True, the execution of a benchmark
-                                     scenario will stop when any SLA check
-                                     for it fails
+        :param abort_on_sla_failure: If set to True, the task execution will
+                                     stop when any SLA check for it fails
         """
         deployment = objects.Deployment.get(deployment)
         task = task or objects.Task(deployment_uuid=deployment["uuid"])
@@ -225,7 +294,7 @@ class Task(object):
 
         LOG.info("Benchmark Task %s on Deployment %s" % (task["uuid"],
                                                          deployment["uuid"]))
-        benchmark_engine = engine.BenchmarkEngine(
+        benchmark_engine = engine.TaskEngine(
             config, task, admin=deployment["admin"], users=deployment["users"],
             abort_on_sla_failure=abort_on_sla_failure)
 
@@ -241,7 +310,7 @@ class Task(object):
 
         :param task_uuid: The UUID of the task
         :type task_uuid: str
-        :param soft: if set to True, task should be aborted after execution of
+        :param soft: If set to True, task should be aborted after execution of
                      current scenario, otherwise as soon as possible before
                      all the scenario iterations finish [Default: False]
         :type soft: bool
@@ -274,12 +343,12 @@ class Task(object):
     def delete(cls, task_uuid, force=False):
         """Delete the task.
 
-        :param task_uuid: The UUID of the task.
+        :param task_uuid: The UUID of the task
         :param force: If set to True, then delete the task despite to the
-                      status.
-        :raises: :class:`rally.exceptions.TaskInvalidStatus` when the
-                 status of the task is not FINISHED and the force argument
-                 if not True
+                      status
+        :raises TaskInvalidStatus: when the status of the task is not
+                                   FINISHED and the force argument
+                                   is not True
         """
         status = None if force else consts.TaskStatus.FINISHED
         objects.Task.delete_by_uuid(task_uuid, status=status)
@@ -288,46 +357,34 @@ class Task(object):
 class Verification(object):
 
     @classmethod
-    def verify(cls, deployment, set_name, regex, tempest_config,
-               system_wide_install=False):
-        """Start verifying.
+    def verify(cls, deployment, set_name="", regex=None, tests_file=None,
+               tempest_config=None, expected_failures=None, system_wide=False,
+               concur=0):
+        """Start verification.
 
-        :param deployment: UUID or name of a deployment.
-        :param set_name: Valid name of tempest test set.
+        :param deployment: UUID or name of a deployment
+        :param set_name: Name of a Tempest test set
         :param regex: Regular expression of test
-        :param tempest_config: User specified Tempest config file
-        :param system_wide_install: Use virtualenv else run tests in local
-                                    environment
+        :param tests_file: Path to a file with a list of Tempest tests
+        :param tempest_config: User specified Tempest config file location
+        :param expected_failures: Dictionary with Tempest tests that are
+                                  expected to fail. Keys are test names;
+                                  values are reasons of test failures
+        :param system_wide: Whether or not to create a virtual env when
+                            installing Tempest; whether or not to use
+                            the local env instead of the Tempest virtual
+                            env when running the tests
+        :param concur: How many processes to use to run Tempest tests.
+                       The default value (0) auto-detects CPU count
         :returns: Verification object
         """
-
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
-
         verification = objects.Verification(deployment_uuid=deployment_uuid)
-        verifier = cls._create_verifier(deployment_uuid, verification,
-                                        tempest_config, system_wide_install)
-        LOG.info("Starting verification of deployment: %s" % deployment_uuid)
-
-        verification.set_running()
-        verifier.verify(set_name=set_name, regex=regex)
-
-        return verification
-
-    @staticmethod
-    def _create_verifier(deployment_uuid, verification=None,
-                         tempest_config=None, system_wide_install=False):
-        """Create a Tempest object.
-
-        :param deployment_uuid: UUID or name of a deployment
-        :param verification: Verification object
-        :param tempest_config: User specified Tempest config file
-        :param system_wide_install: Use virtualenv else run tests in local
-                                    environment
-        :return: Tempest object
-        """
-        verifier = tempest.Tempest(deployment_uuid, verification=verification,
+        verifier = tempest.Tempest(deployment_uuid,
+                                   verification=verification,
                                    tempest_config=tempest_config,
-                                   system_wide_install=system_wide_install)
+                                   system_wide=system_wide)
+
         if not verifier.is_installed():
             LOG.info(_("Tempest is not installed "
                        "for the specified deployment."))
@@ -335,7 +392,12 @@ class Verification(object):
                        "for deployment: %s") % deployment_uuid)
             verifier.install()
 
-        return verifier
+        LOG.info("Starting verification of deployment: %s" % deployment_uuid)
+        verification.set_running()
+        verifier.verify(set_name=set_name, regex=regex, tests_file=tests_file,
+                        expected_failures=expected_failures, concur=concur)
+
+        return verification
 
     @classmethod
     def import_results(cls, deployment, set_name="", log_file=None):
@@ -345,50 +407,51 @@ class Verification(object):
         :param log_file: User specified Tempest log file in subunit format
         :returns: Deployment and verification objects
         """
-
         # TODO(aplanas): Create an external deployment if this is
         # missing, as required in the blueprint [1].
         # [1] https://blueprints.launchpad.net/rally/+spec/verification-import
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
-
         verification = objects.Verification(deployment_uuid=deployment_uuid)
         verifier = tempest.Tempest(deployment_uuid, verification=verification)
-        LOG.info("Importing verification of deployment: %s" % deployment_uuid)
 
+        LOG.info("Importing verification of deployment: %s" % deployment_uuid)
         verification.set_running()
         verifier.import_results(set_name=set_name, log_file=log_file)
 
         return deployment, verification
 
     @classmethod
-    def install_tempest(cls, deployment, source=None):
+    def install_tempest(cls, deployment, source=None, no_tempest_venv=False):
         """Install Tempest.
 
-        :param deployment: UUID or name of the deployment
-        :param source: Source to fetch Tempest from
+        :param deployment: UUID or name of a deployment
+        :param source: Path/URL to repo to clone Tempest from
+        :param no_tempest_venv: Whether or not to create a Tempest virtual env
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
-        verifier = tempest.Tempest(deployment_uuid, source=source)
+        verifier = tempest.Tempest(deployment_uuid, source=source,
+                                   system_wide=no_tempest_venv)
         verifier.install()
 
     @classmethod
     def uninstall_tempest(cls, deployment):
         """Remove deployment's local Tempest installation.
 
-        :param deployment: UUID or name of the deployment
+        :param deployment: UUID or name of a deployment
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
         verifier = tempest.Tempest(deployment_uuid)
         verifier.uninstall()
 
     @classmethod
-    def reinstall_tempest(cls, deployment, tempest_config=None, source=None):
-        """Uninstall Tempest and then reinstall from new source.
+    def reinstall_tempest(cls, deployment, tempest_config=None,
+                          source=None, no_tempest_venv=False):
+        """Uninstall Tempest and install again.
 
-        :param deployment: UUID or name of the deployment
-        :param tempest_config: Tempest config file. Use previous file as
-        default
-        :param source: Source to fetch Tempest from. Use old source as default
+        :param deployment: UUID or name of a deployment
+        :param tempest_config: User specified Tempest config file location
+        :param source: Path/URL to repo to clone Tempest from
+        :param no_tempest_venv: Whether or not to create a Tempest virtual env
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
         verifier = tempest.Tempest(deployment_uuid)
@@ -401,11 +464,21 @@ class Verification(object):
         source = source or verifier.tempest_source
         verifier.uninstall()
         verifier = tempest.Tempest(deployment_uuid, source=source,
-                                   tempest_config=tempest_config)
+                                   tempest_config=tempest_config,
+                                   system_wide=no_tempest_venv)
         verifier.install()
         if not tempest_config:
             shutil.move(tmp_conf_path, verifier.config_file)
 
+    @staticmethod
+    def _check_tempest_tree_existence(verifier):
+        if not os.path.exists(verifier.path()):
+            msg = _("Tempest tree for "
+                    "deployment '%s' not found! ") % verifier.deployment
+            LOG.error(
+                msg + _("Use `rally verify install` for Tempest installation"))
+            raise exceptions.NotFoundException(message=msg)
+
     @classmethod
     def configure_tempest(cls, deployment, tempest_config=None,
                           override=False):
@@ -413,9 +486,47 @@ class Verification(object):
 
         :param deployment: UUID or name of a deployment
         :param tempest_config: User specified Tempest config file location
-        :param override: Whether or not override existing Tempest config file
+        :param override: Whether or not to override existing Tempest
+                         config file
         """
         deployment_uuid = objects.Deployment.get(deployment)["uuid"]
-        verifier = cls._create_verifier(deployment_uuid,
-                                        tempest_config=tempest_config)
+        verifier = tempest.Tempest(deployment_uuid,
+                                   tempest_config=tempest_config)
+
+        cls._check_tempest_tree_existence(verifier)
+
         verifier.generate_config_file(override)
+
+    @classmethod
+    def show_config_info(cls, deployment):
+        """Show information about configuration file of Tempest.
+
+        :param deployment: UUID or name of a deployment
+        """
+        deployment_uuid = objects.Deployment.get(deployment)["uuid"]
+        verifier = tempest.Tempest(deployment_uuid)
+
+        cls._check_tempest_tree_existence(verifier)
+
+        if not verifier.is_configured():
+            verifier.generate_config_file()
+
+        with open(verifier.config_file, "rb") as conf:
+            return {"conf_data": conf.read(),
+                    "conf_path": verifier.config_file}
+
+    @staticmethod
+    def list(status=None):
+        """List all verifications.
+
+        :param status: Filter verifications by the specified status
+        """
+        return objects.Verification.list(status)
+
+    @staticmethod
+    def get(verification_uuid):
+        """Get verification.
+
+        :param verification_uuid: UUID of a verification
+        """
+        return objects.Verification.get(verification_uuid)
