@@ -15,11 +15,11 @@
 
 import random
 
-from rally.common import log as logging
+from rally.common import logging
 from rally import consts
 from rally import exceptions
 from rally.plugins.openstack import scenario
-from rally.plugins.openstack.scenarios.cinder import utils
+from rally.plugins.openstack.scenarios.cinder import utils as cinder_utils
 from rally.plugins.openstack.scenarios.glance import utils as glance_utils
 from rally.plugins.openstack.scenarios.nova import utils as nova_utils
 from rally.task import types
@@ -28,7 +28,7 @@ from rally.task import validation
 LOG = logging.getLogger(__name__)
 
 
-class CinderVolumes(utils.CinderScenario,
+class CinderVolumes(cinder_utils.CinderScenario,
                     nova_utils.NovaScenario,
                     glance_utils.GlanceScenario):
     """Benchmark scenarios for Cinder Volumes."""
@@ -278,7 +278,12 @@ class CinderVolumes(utils.CinderScenario,
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["cinder", "nova"]})
-    def create_and_attach_volume(self, size, image, flavor, **kwargs):
+    @logging.log_deprecated_args(
+        "Use 'create_vm_params' for additional instance parameters.",
+        "0.2.0", ["kwargs"], once=True)
+    def create_and_attach_volume(self, size, image, flavor,
+                                 create_volume_params=None,
+                                 create_vm_params=None, **kwargs):
         """Create a VM and attach a volume to it.
 
         Simple test to create a VM and attach a volume, then
@@ -290,11 +295,22 @@ class CinderVolumes(utils.CinderScenario,
                          max - maximum size volumes will be created as.
         :param image: Glance image name to use for the VM
         :param flavor: VM flavor name
-        :param kwargs: optional arguments for VM creation
+        :param create_volume_params: optional arguments for volume creation
+        :param create_vm_params: optional arguments for VM creation
+        :param kwargs: (deprecated) optional arguments for VM creation
         """
 
-        server = self._boot_server(image, flavor, **kwargs)
-        volume = self._create_volume(size)
+        create_volume_params = create_volume_params or {}
+
+        if kwargs and create_vm_params:
+            raise ValueError("You can not set both 'kwargs'"
+                             "and 'create_vm_params' attributes."
+                             "Please use 'create_vm_params'.")
+
+        create_vm_params = create_vm_params or kwargs or {}
+
+        server = self._boot_server(image, flavor, **create_vm_params)
+        volume = self._create_volume(size, **create_volume_params)
 
         self._attach_volume(server, volume)
         self._detach_volume(server, volume)
@@ -311,7 +327,7 @@ class CinderVolumes(utils.CinderScenario,
 
         """Create volume, snapshot and attach/detach volume.
 
-        This scenario is based off of the standalone qaStressTest.py
+        This scenario is based on the standalone qaStressTest.py
         (https://github.com/WaltHP/cinder-stress).
 
         :param volume_type: Whether or not to specify volume type when creating
@@ -348,6 +364,8 @@ class CinderVolumes(utils.CinderScenario,
     @validation.required_services(consts.Service.NOVA, consts.Service.CINDER)
     @validation.required_openstack(users=True)
     @scenario.configure(context={"cleanup": ["cinder", "nova"]})
+    @logging.log_deprecated_args("Use 'nested_level' as an int", "0.1.2",
+                                 ["nested_level"], once=True)
     def create_nested_snapshots_and_attach_volume(self,
                                                   size=None,
                                                   nested_level=None,
@@ -364,19 +382,25 @@ class CinderVolumes(utils.CinderScenario,
                         min - minimum size volumes will be created as;
                         max - maximum size volumes will be created as.
                      default values: {"min": 1, "max": 5}
-        :param nested_level: Nested level - dictionary, contains two values:
+        :param nested_level: Nested level - dictionary or int, dictionary
+                             contains two values:
                                min - minimum number of volumes will be created
                                      from snapshot;
                                max - maximum number of volumes will be created
                                      from snapshot.
-                             default values: {"min": 5, "max": 10}
+                             due to its deprecated would be taken min value.
+                             int, means the exact nested level.
+                             default value: 1.
         :param kwargs: Optional parameters used during volume
                        snapshot creation.
         """
         if size is None:
             size = {"min": 1, "max": 5}
         if nested_level is None:
-            nested_level = {"min": 5, "max": 10}
+            nested_level = 1
+        nested_level = nested_level or 1
+        if isinstance(nested_level, dict):
+            nested_level = nested_level.get("min", 1)
 
         # NOTE: Volume size cannot be smaller than the snapshot size, so
         #       volume with specified size should be created to avoid
@@ -384,8 +408,6 @@ class CinderVolumes(utils.CinderScenario,
         #       size in _create_volume method.
         size = random.randint(size["min"], size["max"])
 
-        nested_level = random.randint(nested_level["min"], nested_level["max"])
-
         source_vol = self._create_volume(size)
         nes_objs = [(self.get_random_server(), source_vol,
                      self._create_snapshot(source_vol.id, False, **kwargs))]
