@@ -15,19 +15,21 @@
 
 import inspect
 import os
-import uuid
 
 from oslo_config import cfg
 import requests
+import six
 from six.moves import configparser
 from six.moves.urllib import parse
 
 from rally.common import db
 from rally.common.i18n import _
-from rally.common import log as logging
+from rally.common import logging
 from rally.common import objects
+from rally.common import utils
 from rally import exceptions
 from rally import osclients
+from rally.plugins.openstack.wrappers import network
 
 LOG = logging.getLogger(__name__)
 
@@ -35,7 +37,13 @@ IMAGE_OPTS = [
     cfg.StrOpt("cirros_img_url",
                default="http://download.cirros-cloud.net/"
                        "0.3.4/cirros-0.3.4-x86_64-disk.img",
-               help="CirrOS image URL")
+               help="CirrOS image URL"),
+    cfg.StrOpt("disk_format",
+               default="qcow2",
+               help="Image disk format"),
+    cfg.StrOpt("container_format",
+               default="bare",
+               help="Image container formate")
 ]
 
 ROLE_OPTS = [
@@ -59,8 +67,6 @@ CONF = cfg.CONF
 CONF.register_opts(IMAGE_OPTS, "image")
 CONF.register_opts(ROLE_OPTS, "role")
 
-IMAGE_NAME = parse.urlparse(CONF.image.cirros_img_url).path.split("/")[-1]
-
 
 def _create_or_get_data_dir():
     data_dir = os.path.join(
@@ -76,27 +82,28 @@ def _write_config(conf_path, conf_data):
         conf_data.write(conf_file)
 
 
-class TempestConfig(object):
+class TempestConfig(utils.RandomNameGeneratorMixin):
     """Class to generate Tempest configuration file."""
 
     def __init__(self, deployment):
         self.deployment = deployment
 
-        self.endpoint = db.deployment_get(deployment)["admin"]
-        self.clients = osclients.Clients(objects.Endpoint(**self.endpoint))
+        self.credential = db.deployment_get(deployment)["admin"]
+        self.clients = osclients.Clients(objects.Credential(**self.credential))
         self.keystone = self.clients.verified_keystone()
-
         self.available_services = self.clients.services().values()
 
         self.data_dir = _create_or_get_data_dir()
 
         self.conf = configparser.ConfigParser()
         self.conf.read(os.path.join(os.path.dirname(__file__), "config.ini"))
+        self.image_name = parse.urlparse(
+            CONF.image.cirros_img_url).path.split("/")[-1]
 
         self._download_cirros_image()
 
     def _download_cirros_image(self):
-        img_path = os.path.join(self.data_dir, IMAGE_NAME)
+        img_path = os.path.join(self.data_dir, self.image_name)
         if os.path.isfile(img_path):
             return
 
@@ -129,6 +136,11 @@ class TempestConfig(object):
             if self.clients.services().get(service["type"]) == service_type:
                 return service["endpoints"][0]["publicURL"]
 
+    def _get_service_type_by_service_name(self, service_name):
+        for s_type, s_name in six.iteritems(self.clients.services()):
+            if s_name == service_name:
+                return s_type
+
     def _configure_boto(self, section_name="boto"):
         self.conf.set(section_name, "ec2_url", self._get_service_url("ec2"))
         self.conf.set(section_name, "s3_url", self._get_service_url("s3"))
@@ -141,29 +153,48 @@ class TempestConfig(object):
         pass
 
     def _configure_dashboard(self, section_name="dashboard"):
-        url = "http://%s/" % parse.urlparse(self.endpoint["auth_url"]).hostname
+        url = "http://%s/" % parse.urlparse(
+            self.credential["auth_url"]).hostname
         self.conf.set(section_name, "dashboard_url", url)
 
+    # Sahara has two service types: 'data_processing' and 'data-processing'.
+    # 'data_processing' is deprecated, but it can be used in previous OpenStack
+    # releases. So we need to configure the 'catalog_type' option to support
+    # environments where 'data_processing' is used as service type for Sahara.
+    def _configure_data_processing(self, section_name="data-processing"):
+        if "sahara" in self.available_services:
+            self.conf.set(section_name, "catalog_type",
+                          self._get_service_type_by_service_name("sahara"))
+
     def _configure_identity(self, section_name="identity"):
-        self.conf.set(section_name, "username", self.endpoint["username"])
-        self.conf.set(section_name, "password", self.endpoint["password"])
+        self.conf.set(section_name, "username", self.credential["username"])
+        self.conf.set(section_name, "password", self.credential["password"])
         self.conf.set(section_name, "tenant_name",
-                      self.endpoint["tenant_name"])
+                      self.credential["tenant_name"])
 
         self.conf.set(section_name, "admin_username",
-                      self.endpoint["username"])
+                      self.credential["username"])
         self.conf.set(section_name, "admin_password",
-                      self.endpoint["password"])
+                      self.credential["password"])
         self.conf.set(section_name, "admin_tenant_name",
-                      self.endpoint["tenant_name"])
+                      self.credential["tenant_name"])
 
-        self.conf.set(section_name, "uri", self.endpoint["auth_url"])
-        v2_url_trailer = parse.urlparse(self.endpoint["auth_url"]).path
+        self.conf.set(section_name, "region",
+                      self.credential["region_name"])
+
+        self.conf.set(section_name, "uri", self.credential["auth_url"])
+        v2_url_trailer = parse.urlparse(self.credential["auth_url"]).path
         self.conf.set(section_name, "uri_v3",
-                      self.endpoint["auth_url"].replace(v2_url_trailer, "/v3"))
+                      self.credential["auth_url"].replace(
+                          v2_url_trailer, "/v3"))
 
         self.conf.set(section_name, "admin_domain_name",
-                      self.endpoint["admin_domain_name"])
+                      self.credential["admin_domain_name"])
+
+        self.conf.set(section_name, "disable_ssl_certificate_validation",
+                      str(self.credential["https_insecure"]))
+        self.conf.set(section_name, "ca_certificates_file",
+                      self.credential["https_cacert"])
 
     # The compute section is configured in context class for Tempest resources.
     # Options which are configured there: 'image_ref', 'image_ref_alt',
@@ -210,7 +241,7 @@ class TempestConfig(object):
 
     def _configure_scenario(self, section_name="scenario"):
         self.conf.set(section_name, "img_dir", self.data_dir)
-        self.conf.set(section_name, "img_file", IMAGE_NAME)
+        self.conf.set(section_name, "img_file", self.image_name)
 
     def _configure_service_available(self, section_name="service_available"):
         services = ["ceilometer", "cinder", "glance",
@@ -221,7 +252,7 @@ class TempestConfig(object):
             self.conf.set(section_name, service,
                           str(service in self.available_services))
         horizon_url = ("http://" +
-                       parse.urlparse(self.endpoint["auth_url"]).hostname)
+                       parse.urlparse(self.credential["auth_url"]).hostname)
         try:
             horizon_req = requests.get(
                 horizon_url,
@@ -241,6 +272,12 @@ class TempestConfig(object):
         else:
             self.conf.set(section_name, "connect_method", "fixed")
 
+    def _configure_orchestration(self, section_name="orchestration"):
+        self.conf.set(section_name, "stack_owner_role",
+                      CONF.role.heat_stack_owner_role)
+        self.conf.set(section_name, "stack_user_role",
+                      CONF.role.heat_stack_user_role)
+
     def generate(self, conf_path=None):
         for name, method in inspect.getmembers(self, inspect.ismethod):
             if name.startswith("_configure_"):
@@ -250,56 +287,96 @@ class TempestConfig(object):
             _write_config(conf_path, self.conf)
 
 
-class TempestResourcesContext(object):
+class TempestResourcesContext(utils.RandomNameGeneratorMixin):
     """Context class to create/delete resources needed for Tempest."""
 
-    def __init__(self, deployment, conf_path):
-        endpoint = db.deployment_get(deployment)["admin"]
-        self.clients = osclients.Clients(objects.Endpoint(**endpoint))
-        self.keystone = self.clients.verified_keystone()
+    RESOURCE_NAME_FORMAT = "rally_verify_XXXXXXXX_XXXXXXXX"
+
+    def __init__(self, deployment, verification, conf_path):
+        credential = db.deployment_get(deployment)["admin"]
+        self.clients = osclients.Clients(objects.Credential(**credential))
+        self.available_services = self.clients.services().values()
+
+        self.verification = verification
 
         self.conf_path = conf_path
         self.conf = configparser.ConfigParser()
         self.conf.read(conf_path)
+        self.image_name = parse.urlparse(
+            CONF.image.cirros_img_url).path.split("/")[-1]
 
-    def __enter__(self):
         self._created_roles = []
         self._created_images = []
         self._created_flavors = []
+        self._created_networks = []
 
+    def __enter__(self):
         self._create_tempest_roles()
-        self._configure_option("image_ref", self._create_image)
-        self._configure_option("image_ref_alt", self._create_image)
-        self._configure_option("flavor_ref", self._create_flavor, 64)
-        self._configure_option("flavor_ref_alt", self._create_flavor, 128)
+        self._configure_option("compute", "image_ref", self._create_image)
+        self._configure_option("compute", "image_ref_alt", self._create_image)
+        self._configure_option("compute",
+                               "flavor_ref", self._create_flavor, 64)
+        self._configure_option("compute",
+                               "flavor_ref_alt", self._create_flavor, 128)
+        if "neutron" in self.available_services:
+            neutronclient = self.clients.neutron()
+            if neutronclient.list_networks(shared=True)["networks"]:
+                # If the OpenStack cloud has some shared networks, we will
+                # create our own shared network and specify its name in the
+                # Tempest config file. Such approach will allow us to avoid
+                # failures of Tempest tests with error "Multiple possible
+                # networks found". Otherwise the default behavior defined in
+                # Tempest will be used and Tempest itself will manage network
+                # resources.
+                LOG.debug("Shared networks found. "
+                          "'fixed_network_name' option should be configured")
+                self._configure_option("compute", "fixed_network_name",
+                                       self._create_network_resources)
+        if "heat" in self.available_services:
+            self._configure_option("orchestration", "instance_type",
+                                   self._create_flavor, 64)
 
         _write_config(self.conf_path, self.conf)
 
     def __exit__(self, exc_type, exc_value, exc_traceback):
-        self._cleanup_roles()
-        self._cleanup_resource("image", self._created_images)
-        self._cleanup_resource("flavor", self._created_flavors)
+        # Tempest tests may take more than 1 hour and we should remove all
+        # cached clients sessions to avoid tokens expiration when deleting
+        # Tempest resources.
+        self.clients.clear()
+
+        self._cleanup_tempest_roles()
+        self._cleanup_images()
+        self._cleanup_flavors()
+        if "neutron" in self.available_services:
+            self._cleanup_network_resources()
+
+        _write_config(self.conf_path, self.conf)
 
     def _create_tempest_roles(self):
+        keystoneclient = self.clients.verified_keystone()
         roles = [CONF.role.swift_operator_role,
                  CONF.role.swift_reseller_admin_role,
                  CONF.role.heat_stack_owner_role,
                  CONF.role.heat_stack_user_role]
-        existing_roles = set(role.name for role in self.keystone.roles.list())
+        existing_roles = set(role.name for role in keystoneclient.roles.list())
 
         for role in roles:
             if role not in existing_roles:
                 LOG.debug("Creating role '%s'" % role)
-                self._created_roles.append(self.keystone.roles.create(role))
+                self._created_roles.append(keystoneclient.roles.create(role))
 
-    def _configure_option(self, option, create_method, *args, **kwargs):
-        option_value = self.conf.get("compute", option)
+    def _configure_option(self, section, option,
+                          create_method, *args, **kwargs):
+        option_value = self.conf.get(section, option)
         if not option_value:
-            LOG.debug("Option '%s' is not configured" % option)
+            LOG.debug("Option '%s' from '%s' section "
+                      "is not configured" % (option, section))
             resource = create_method(*args, **kwargs)
-            self.conf.set("compute", option, resource.id)
-            LOG.debug("Option '{opt}' is configured. {opt} = {resource_id}"
-                      .format(opt=option, resource_id=resource.id))
+            value = resource["name"] if "network" in option else resource.id
+            LOG.debug("Setting value '%s' for option '%s'" % (value, option))
+            self.conf.set(section, option, value)
+            LOG.debug("Option '{opt}' is configured. "
+                      "{opt} = {value}".format(opt=option, value=value))
         else:
             LOG.debug("Option '{opt}' was configured manually "
                       "in Tempest config file. {opt} = {opt_val}"
@@ -307,52 +384,75 @@ class TempestResourcesContext(object):
 
     def _create_image(self):
         glanceclient = self.clients.glance()
-        image_name = "rally-verify-cirros-img-%s" % uuid.uuid4()
-        LOG.debug("Creating image '%s'" % image_name)
-        try:
-            image = glanceclient.images.create(
-                name=image_name,
-                disk_format="qcow2",
-                container_format="bare",
-                is_public=True)
-            self._created_images.append(image)
-            image.update(data=open(
-                os.path.join(_create_or_get_data_dir(), IMAGE_NAME), "rb"))
-        except Exception as exc:
-            msg = _("Image could not be created. "
-                    "Reason: %s") % (str(exc) or "unknown")
-            raise exceptions.TempestResourceCreationFailure(msg)
+        params = {
+            "name": self.generate_random_name(),
+            "disk_format": CONF.image.disk_format,
+            "container_format": CONF.image.container_format,
+            "is_public": True
+        }
+        LOG.debug("Creating image '%s'" % params["name"])
+        image = glanceclient.images.create(**params)
+        self._created_images.append(image)
+        image.update(data=open(
+            os.path.join(_create_or_get_data_dir(), self.image_name), "rb"))
 
         return image
 
     def _create_flavor(self, flv_ram):
         novaclient = self.clients.nova()
-        flavor_name = "m1.rally-verify-flv-%s" % uuid.uuid4()
-        LOG.debug("Creating flavor '%s'" % flavor_name)
-        try:
-            flavor = novaclient.flavors.create(
-                flavor_name, ram=flv_ram, vcpus=1, disk=0)
-        except Exception as exc:
-            msg = _("Flavor could not be created. "
-                    "Reason: %s") % (str(exc) or "unknown")
-            raise exceptions.TempestResourceCreationFailure(msg)
-
+        params = {
+            "name": self.generate_random_name(),
+            "ram": flv_ram,
+            "vcpus": 1,
+            "disk": 0
+        }
+        LOG.debug("Creating flavor '%s'" % params["name"])
+        flavor = novaclient.flavors.create(**params)
         self._created_flavors.append(flavor)
 
         return flavor
 
-    def _cleanup_roles(self):
+    def _create_network_resources(self):
+        neutron_wrapper = network.NeutronWrapper(self.clients, self)
+        LOG.debug("Creating network resources: network, subnet, router")
+        net = neutron_wrapper.create_network(
+            self.clients.keystone().tenant_id, subnets_num=1,
+            add_router=True, network_create_args={"shared": True})
+        self._created_networks.append(net)
+
+        return net
+
+    def _cleanup_tempest_roles(self):
+        keystoneclient = self.clients.keystone()
         for role in self._created_roles:
             LOG.debug("Deleting role '%s'" % role.name)
-            role.delete()
+            keystoneclient.roles.delete(role.id)
 
-    def _cleanup_resource(self, resource_type, created_resources):
-        for res in created_resources:
-            LOG.debug("Deleting %s '%s'" % (resource_type, res.name))
-            if res.id == self.conf.get("compute", "%s_ref" % resource_type):
-                self.conf.set("compute", "%s_ref" % resource_type, "")
-            else:
-                self.conf.set("compute", "%s_ref_alt" % resource_type, "")
-            res.delete()
+    def _cleanup_images(self):
+        glanceclient = self.clients.glance()
+        for image in self._created_images:
+            LOG.debug("Deleting image '%s'" % image.name)
+            glanceclient.images.delete(image.id)
+            self._remove_opt_value_from_config("compute", image.id)
 
-        _write_config(self.conf_path, self.conf)
+    def _cleanup_flavors(self):
+        novaclient = self.clients.nova()
+        for flavor in self._created_flavors:
+            LOG.debug("Deleting flavor '%s'" % flavor.name)
+            novaclient.flavors.delete(flavor.id)
+            self._remove_opt_value_from_config("compute", flavor.id)
+            self._remove_opt_value_from_config("orchestration", flavor.id)
+
+    def _cleanup_network_resources(self):
+        neutron_wrapper = network.NeutronWrapper(self.clients, self)
+        for net in self._created_networks:
+            LOG.debug("Deleting network resources: router, subnet, network")
+            neutron_wrapper.delete_network(net)
+            self._remove_opt_value_from_config("compute", net["name"])
+
+    def _remove_opt_value_from_config(self, section, opt_value):
+        for option, value in self.conf.items(section):
+            if opt_value == value:
+                LOG.debug("Removing value '%s' for option '%s' "
+                          "from Tempest config file" % (opt_value, option))
+                self.conf.set(section, option, "")
