@@ -18,10 +18,8 @@ import os.path
 import tempfile
 
 import mock
-import six
 
 from rally.cli.commands import verify
-from rally.common import objects
 from rally import consts
 from rally import exceptions
 from tests.unit import test
@@ -56,11 +54,11 @@ class VerifyCommandsTestCase(test.TestCase):
             self.flavor1, self.flavor2]
 
         self.verify.start(deployment=deployment_id, do_use=False)
-        default_set_name = "full"
-        default_regex = None
 
         mock_verification_verify.assert_called_once_with(
-            deployment_id, default_set_name, default_regex, None, False)
+            deployment_id, set_name="full", regex=None, tests_file=None,
+            tempest_config=None, expected_failures=None, system_wide=False,
+            concur=0)
 
     @mock.patch("rally.osclients.Clients")
     @mock.patch("rally.api.Verification.verify")
@@ -74,21 +72,50 @@ class VerifyCommandsTestCase(test.TestCase):
         tempest_config = tempfile.NamedTemporaryFile()
         self.verify.start(deployment=deployment_id,
                           tempest_config=tempest_config.name, do_use=False)
-        default_set_name = "full"
-        default_regex = None
 
         mock_verification_verify.assert_called_once_with(
-            deployment_id, default_set_name, default_regex,
-            tempest_config.name, False)
+            deployment_id, set_name="full", regex=None, tests_file=None,
+            tempest_config=tempest_config.name, expected_failures=None,
+            system_wide=False, concur=0)
         tempest_config.close()
 
+    @mock.patch("rally.api.Verification.verify")
+    @mock.patch("os.path.exists", return_value=True)
+    def test_start_with_tests_file_specified(self, mock_exists,
+                                             mock_verification_verify):
+        deployment_id = "f05645f9-b3d1-4be4-ae63-ae6ea6d89f17"
+        tests_file = "/path/to/tests/file"
+        self.verify.start(deployment=deployment_id,
+                          tests_file=tests_file, do_use=False)
+
+        mock_verification_verify.assert_called_once_with(
+            deployment_id, set_name="", regex=None, tests_file=tests_file,
+            tempest_config=None, expected_failures=None, system_wide=False,
+            concur=0)
+
+    @mock.patch("rally.api.Verification.verify")
+    @mock.patch("six.moves.builtins.open",
+                side_effect=mock.mock_open(read_data="test: reason of fail"))
+    @mock.patch("os.path.exists", return_value=True)
+    def test_start_with_xfails_file_specified(self, mock_exists, mock_open,
+                                              mock_verification_verify):
+        deployment_id = "eba53a0e-e2e6-451c-9a29-bdd2efc245e7"
+        xfails_file = "/path/to/xfails/file"
+        self.verify.start(deployment=deployment_id,
+                          xfails_file=xfails_file, do_use=False)
+
+        mock_verification_verify.assert_called_once_with(
+            deployment_id, set_name="full", regex=None, tests_file=None,
+            tempest_config=None, expected_failures={"test": "reason of fail"},
+            system_wide=False, concur=0)
+
     @mock.patch("rally.api.Verification.verify")
     def test_start_with_wrong_set_name(self, mock_verification_verify):
         deployment_id = "f2009aae-6ef3-468e-96b2-3c987d584010"
 
         wrong_set_name = "unexpected_value"
 
-        self.verify.start(deployment_id, wrong_set_name, do_use=False)
+        self.verify.start(deployment_id, set_name=wrong_set_name, do_use=False)
 
         self.assertNotIn(wrong_set_name, consts.TempestTestsSets,
                          consts.TempestTestsAPI)
@@ -119,122 +146,113 @@ class VerifyCommandsTestCase(test.TestCase):
             deployment_id, set_name, log_file)
 
     @mock.patch("rally.cli.cliutils.print_list")
-    @mock.patch("rally.common.db.verification_list")
-    def test_list(self, mock_common_db_verification_list, mock_print_list):
+    @mock.patch("rally.api.Verification.list")
+    def test_list(self, mock_verification_list, mock_print_list):
         fields = ["UUID", "Deployment UUID", "Set name", "Tests", "Failures",
                   "Created at", "Duration", "Status"]
         verifications = [{"created_at": date.datetime.now(),
                           "updated_at": date.datetime.now()}]
-        mock_common_db_verification_list.return_value = verifications
+        mock_verification_list.return_value = verifications
         self.verify.list()
 
         for row in verifications:
             self.assertEqual(row["updated_at"] - row["created_at"],
                              row["duration"])
 
-        mock_common_db_verification_list.assert_called_once_with()
+        mock_verification_list.assert_called_once_with()
         mock_print_list.assert_called_once_with(verifications, fields,
                                                 sortby_index=fields.index(
                                                     "Created at"))
 
     @mock.patch("rally.cli.cliutils.print_list")
-    @mock.patch("rally.common.db.verification_get")
-    @mock.patch("rally.common.db.verification_result_get")
-    @mock.patch("rally.common.objects.Verification")
-    def test_show(self, mock_objects_verification,
-                  mock_verification_result_get, mock_verification_get,
-                  mock_print_list):
+    @mock.patch("rally.common.utils.Struct")
+    @mock.patch("rally.api.Verification")
+    def test_show(self, mock_verification, mock_struct, mock_print_list):
+        verification = mock_verification.get.return_value
 
-        class Test_dummy():
-            data = {"test_cases": {"test_a": {"name": "test_a", "time": 20,
-                                              "status": "PASS"},
-                                   "test_b": {"name": "test_b", "time": 20,
-                                              "status": "SKIP"},
-                                   "test_c": {"name": "test_c", "time": 20,
-                                              "status": "FAIL"}}}
+        tests = {"test_cases": {"test_a": {"name": "test_a", "time": 20,
+                                           "status": "success"},
+                                "test_b": {"name": "test_b", "time": 20,
+                                           "status": "skip"},
+                                "test_c": {"name": "test_c", "time": 20,
+                                           "status": "fail"}}}
 
         verification_id = "39121186-b9a4-421d-b094-6c6b270cf9e9"
         total_fields = ["UUID", "Deployment UUID", "Set name", "Tests",
                         "Failures", "Created at", "Status"]
         fields = ["name", "time", "status"]
-        verification = mock.MagicMock()
-        tests = Test_dummy()
-        mock_verification_result_get.return_value = tests
-        mock_verification_get.return_value = verification
-        mock_objects_verification.return_value = 1
-        values = [objects.Verification(t)
-                  for t in six.itervalues(tests.data["test_cases"])]
-        self.verify.show(verification_id)
-        self.assertEqual([mock.call([verification], fields=total_fields),
-                          mock.call(values, fields, sortby_index=0)],
-                         mock_print_list.call_args_list)
-        mock_verification_get.assert_called_once_with(verification_id)
-        mock_verification_result_get.assert_called_once_with(verification_id)
+        verification.get_results.return_value = tests
+        values = [mock_struct(), mock_struct(), mock_struct()]
 
-    @mock.patch("rally.common.db.verification_result_get",
-                return_value={"data": {}})
+        self.verify.show(verification_id)
+
+        self.assertEqual([mock.call([verification], fields=total_fields),
+                         mock.call(values, fields, sortby_index=0)],
+                         mock_print_list.call_args_list)
+        mock_verification.get.assert_called_once_with(verification_id)
+
+    @mock.patch("rally.api.Verification")
     @mock.patch("json.dumps")
-    def test_results(self, mock_json_dumps, mock_verification_result_get):
+    def test_results(self, mock_json_dumps, mock_verification):
+        mock_verification.get.return_value.get_results.return_value = {}
         verification_uuid = "a0231bdf-6a4e-4daf-8ab1-ae076f75f070"
         self.verify.results(verification_uuid, output_html=False,
                             output_json=True)
 
-        mock_verification_result_get.assert_called_once_with(verification_uuid)
+        mock_verification.get.assert_called_once_with(verification_uuid)
         mock_json_dumps.assert_called_once_with({}, sort_keys=True, indent=4)
 
-    @mock.patch("rally.common.db.verification_result_get")
+    @mock.patch("rally.api.Verification.get")
     def test_results_verification_not_found(
-            self, mock_verification_result_get):
+            self, mock_verification_get):
         verification_uuid = "9044ced5-9c84-4666-8a8f-4b73a2b62acb"
-        mock_verification_result_get.side_effect = (
+        mock_verification_get.side_effect = (
             exceptions.NotFoundException()
         )
         self.assertEqual(self.verify.results(verification_uuid,
                                              output_html=False,
                                              output_json=True), 1)
 
-        mock_verification_result_get.assert_called_once_with(verification_uuid)
+        mock_verification_get.assert_called_once_with(verification_uuid)
 
     @mock.patch("rally.cli.commands.verify.open",
                 side_effect=mock.mock_open(), create=True)
-    @mock.patch("rally.common.db.verification_result_get",
-                return_value={"data": {}})
+    @mock.patch("rally.api.Verification")
     def test_results_with_output_json_and_output_file(
-            self, mock_verification_result_get, mock_open):
+            self, mock_verification, mock_open):
+        mock_verification.get.return_value.get_results.return_value = {}
         mock_open.side_effect = mock.mock_open()
         verification_uuid = "94615cd4-ff45-4123-86bd-4b0741541d09"
         self.verify.results(verification_uuid, output_file="results",
                             output_html=False, output_json=True)
 
-        mock_verification_result_get.assert_called_once_with(verification_uuid)
+        mock_verification.get.assert_called_once_with(verification_uuid)
         mock_open.assert_called_once_with("results", "wb")
         mock_open.side_effect().write.assert_called_once_with("{}")
 
     @mock.patch("rally.cli.commands.verify.open",
                 side_effect=mock.mock_open(), create=True)
-    @mock.patch("rally.common.db.verification_result_get")
-    @mock.patch("rally.verification.tempest.json2html.HtmlOutput")
+    @mock.patch("rally.api.Verification")
+    @mock.patch("rally.verification.tempest.json2html.generate_report")
     def test_results_with_output_html_and_output_file(
-            self, mock_html_output, mock_verification_result_get, mock_open):
+            self, mock_generate_report, mock_verification, mock_open):
 
         verification_uuid = "7140dd59-3a7b-41fd-a3ef-5e3e615d7dfa"
-        fake_data = {}
-        results = {"data": fake_data}
-        mock_verification_result_get.return_value = results
-        mock_create = mock.Mock(return_value="html_report")
-        mock_html_output.return_value = mock.Mock(create_report=mock_create)
         self.verify.results(verification_uuid, output_html=True,
                             output_json=False, output_file="results")
 
-        mock_verification_result_get.assert_called_once_with(verification_uuid)
-        mock_html_output.assert_called_once_with(fake_data)
+        mock_verification.get.assert_called_once_with(verification_uuid)
+        mock_generate_report.assert_called_once_with(
+            mock_verification.get.return_value.get_results.return_value)
         mock_open.assert_called_once_with("results", "wb")
-        mock_open.side_effect().write.assert_called_once_with("html_report")
+        mock_open.side_effect().write.assert_called_once_with(
+            mock_generate_report.return_value)
 
-    @mock.patch("rally.common.db.verification_result_get",
-                return_value={"data": {"test_cases": {}}})
+    @mock.patch("rally.api.Verification")
     @mock.patch("json.dumps")
-    def test_compare(self, mock_json_dumps, mock_verification_result_get):
+    def test_compare(self, mock_json_dumps, mock_verification):
+        mock_verification.get.return_value.get_results.return_value = {
+            "test_cases": {}}
         uuid1 = "8eda1b10-c8a4-4316-9603-8468ff1d1560"
         uuid2 = "f6ef0a98-1b18-452f-a6a7-922555c2e326"
         self.verify.compare(uuid1, uuid2, output_csv=False, output_html=False,
@@ -243,14 +261,13 @@ class VerifyCommandsTestCase(test.TestCase):
         fake_data = []
         calls = [mock.call(uuid1),
                  mock.call(uuid2)]
-        mock_verification_result_get.assert_has_calls(calls, True)
+        mock_verification.get.assert_has_calls(calls, True)
         mock_json_dumps.assert_called_once_with(fake_data, sort_keys=True,
                                                 indent=4)
 
-    @mock.patch("rally.common.db.verification_result_get",
+    @mock.patch("rally.api.Verification.get",
                 side_effect=exceptions.NotFoundException())
-    def test_compare_verification_not_found(self,
-                                            mock_verification_result_get):
+    def test_compare_verification_not_found(self, mock_verification_get):
         uuid1 = "f7dc82da-31a6-4d40-bbf8-6d366d58960f"
         uuid2 = "2f8a05f3-d310-4f02-aabf-e1165aaa5f9c"
 
@@ -258,14 +275,15 @@ class VerifyCommandsTestCase(test.TestCase):
                                              output_html=False,
                                              output_json=True), 1)
 
-        mock_verification_result_get.assert_called_once_with(uuid1)
+        mock_verification_get.assert_called_once_with(uuid1)
 
     @mock.patch("rally.cli.commands.verify.open",
                 side_effect=mock.mock_open(), create=True)
-    @mock.patch("rally.common.db.verification_result_get",
-                return_value={"data": {"test_cases": {}}})
+    @mock.patch("rally.api.Verification")
     def test_compare_with_output_csv_and_output_file(
-            self, mock_verification_result_get, mock_open):
+            self, mock_verification, mock_open):
+        mock_verification.get.return_value.get_results.return_value = {
+            "test_cases": {}}
 
         fake_string = "Type,Field,Value 1,Value 2,Test Name\r\n"
         uuid1 = "5e744557-4c3a-414f-9afb-7d3d8708028f"
@@ -276,16 +294,18 @@ class VerifyCommandsTestCase(test.TestCase):
 
         calls = [mock.call(uuid1),
                  mock.call(uuid2)]
-        mock_verification_result_get.assert_has_calls(calls, True)
+        mock_verification.get.assert_has_calls(calls, True)
         mock_open.assert_called_once_with("results", "wb")
         mock_open.side_effect().write.assert_called_once_with(fake_string)
 
     @mock.patch("rally.cli.commands.verify.open",
                 side_effect=mock.mock_open(), create=True)
-    @mock.patch("rally.common.db.verification_result_get",
-                return_value={"data": {"test_cases": {}}})
+    @mock.patch("rally.api.Verification")
     def test_compare_with_output_json_and_output_file(
-            self, mock_verification_result_get, mock_open):
+            self, mock_verification, mock_open):
+        mock_verification.get.return_value.get_results.return_value = {
+            "test_cases": {}}
+
         fake_json_string = "[]"
         uuid1 = "0505e33a-738d-4474-a611-9db21547d863"
         uuid2 = "b1908417-934e-481c-8d23-bc0badad39ed"
@@ -295,39 +315,39 @@ class VerifyCommandsTestCase(test.TestCase):
 
         calls = [mock.call(uuid1),
                  mock.call(uuid2)]
-        mock_verification_result_get.assert_has_calls(calls, True)
+        mock_verification.get.assert_has_calls(calls, True)
         mock_open.assert_called_once_with("results", "wb")
         mock_open.side_effect().write.assert_called_once_with(fake_json_string)
 
     @mock.patch("rally.cli.commands.verify.open",
                 side_effect=mock.mock_open(), create=True)
-    @mock.patch("rally.common.db.verification_result_get")
-    @mock.patch(("rally.verification.tempest."
-                 "compare2html.create_report"), return_value="")
+    @mock.patch("rally.api.Verification")
+    @mock.patch("rally.verification.tempest.compare2html.create_report",
+                return_value="")
     def test_compare_with_output_html_and_output_file(
             self, mock_compare2html_create_report,
-            mock_verification_result_get, mock_open):
+            mock_verification, mock_open):
+        mock_verification.get.return_value.get_results.return_value = {
+            "test_cases": {}}
 
         uuid1 = "cdf64228-77e9-414d-9d4b-f65e9d62c61f"
         uuid2 = "39393eec-1b45-4103-8ec1-631edac4b8f0"
-        results = {"data": {"test_cases": {}}}
+
         fake_data = []
         self.verify.compare(uuid1, uuid2,
                             output_file="results",
                             output_csv=False, output_html=True,
                             output_json=False)
-        mock_verification_result_get.return_value = results
         calls = [mock.call(uuid1),
                  mock.call(uuid2)]
-        mock_verification_result_get.assert_has_calls(calls, True)
+        mock_verification.get.assert_has_calls(calls, True)
         mock_compare2html_create_report.assert_called_once_with(fake_data)
 
         mock_open.assert_called_once_with("results", "wb")
         mock_open.side_effect().write.assert_called_once_with("")
 
     @mock.patch("rally.common.fileutils._rewrite_env_file")
-    @mock.patch("rally.cli.commands.verify.db.verification_get",
-                return_value=True)
+    @mock.patch("rally.api.Verification.get")
     def test_use(self, mock_verification_get, mock__rewrite_env_file):
         verification_id = "80422553-5774-44bd-98ac-38bd8c7a0feb"
         self.verify.use(verification_id)
@@ -335,7 +355,7 @@ class VerifyCommandsTestCase(test.TestCase):
             os.path.expanduser("~/.rally/globals"),
             ["RALLY_VERIFICATION=%s\n" % verification_id])
 
-    @mock.patch("rally.cli.commands.verify.db.verification_get")
+    @mock.patch("rally.api.Verification.get")
     def test_use_not_found(self, mock_verification_get):
         verification_id = "ddc3f8ba-082a-496d-b18f-72cdf5c10a14"
         mock_verification_get.side_effect = exceptions.NotFoundException(
@@ -382,7 +402,7 @@ class VerifyCommandsTestCase(test.TestCase):
         deployment_uuid = "d26ebebc-3a5f-4d0d-9021-0c883bd560f5"
         self.verify.install(deployment_uuid)
         mock_verification_install_tempest.assert_called_once_with(
-            deployment_uuid, None)
+            deployment_uuid, None, False)
 
     @mock.patch("rally.api.Verification.install_tempest")
     def test_install_with_source_specified(
@@ -391,7 +411,7 @@ class VerifyCommandsTestCase(test.TestCase):
         source = "/tmp/tempest"
         self.verify.install(deployment_uuid, source)
         mock_verification_install_tempest.assert_called_once_with(
-            deployment_uuid, source)
+            deployment_uuid, source, False)
 
     @mock.patch("rally.api.Verification.uninstall_tempest")
     def test_uninstall(self, mock_verification_uninstall_tempest):
@@ -405,7 +425,7 @@ class VerifyCommandsTestCase(test.TestCase):
         deployment_uuid = "05e0879b-9150-4e42-b6a0-3c6e48197cc1"
         self.verify.reinstall(deployment_uuid)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, None, None)
+            deployment_uuid, None, None, False)
 
     @mock.patch("rally.api.Verification.reinstall_tempest")
     def test_reinstall_with_config_specified(
@@ -414,7 +434,7 @@ class VerifyCommandsTestCase(test.TestCase):
         tempest_conf = "/tmp/tempest.conf"
         self.verify.reinstall(deployment_uuid, tempest_config=tempest_conf)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, tempest_conf, None)
+            deployment_uuid, tempest_conf, None, False)
 
     @mock.patch("rally.api.Verification.reinstall_tempest")
     def test_reinstall_with_source_specified(
@@ -423,7 +443,7 @@ class VerifyCommandsTestCase(test.TestCase):
         source = "/tmp/tempest"
         self.verify.reinstall(deployment_uuid, source=source)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, None, source)
+            deployment_uuid, None, source, False)
 
     @mock.patch("rally.api.Verification.reinstall_tempest")
     def test_reinstall_with_config_and_source_specified(
@@ -433,4 +453,11 @@ class VerifyCommandsTestCase(test.TestCase):
         source = "/tmp/tempest"
         self.verify.reinstall(deployment_uuid, tempest_conf, source)
         mock_verification_reinstall_tempest.assert_called_once_with(
-            deployment_uuid, tempest_conf, source)
+            deployment_uuid, tempest_conf, source, False)
+
+    @mock.patch("rally.api.Verification.show_config_info")
+    def test_showconfig(self, mock_verification_show_config_info):
+        deployment_uuid = "571368f4-20dd-443c-a188-4e931cd2abe6"
+        self.verify.showconfig(deployment_uuid)
+        mock_verification_show_config_info.assert_called_once_with(
+            deployment_uuid)
