@@ -32,7 +32,7 @@ class CinderServersTestCase(test.ScenarioTestCase):
         context = test.get_test_context()
         context.update({
             "user": {"tenant_id": "fake",
-                     "endpoint": mock.MagicMock()},
+                     "credential": mock.MagicMock()},
             "tenant": {"id": "fake", "name": "fake",
                        "volumes": [{"id": "uuid"}],
                        "servers": [1]}})
@@ -196,7 +196,12 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._create_volume = mock.MagicMock(return_value=fake_volume)
         scenario._delete_volume = mock.MagicMock()
 
-        scenario.create_and_attach_volume(10, "img", "0")
+        volume_args = {"some_key": "some_val"}
+        vm_args = {"some_key": "some_val"}
+
+        scenario.create_and_attach_volume(10, "img", "0",
+                                          create_volume_params=volume_args,
+                                          create_vm_params=vm_args)
         scenario._attach_volume.assert_called_once_with(fake_server,
                                                         fake_volume)
         scenario._detach_volume.assert_called_once_with(fake_server,
@@ -350,7 +355,7 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._delete_snapshot = mock.MagicMock()
 
         scenario.create_nested_snapshots_and_attach_volume(
-            nested_level={"min": 2, "max": 2})
+            nested_level=2)
 
         vol_delete_calls = [mock.call(fake_volume2), mock.call(fake_volume1)]
         snap_delete_calls = [mock.call(fake_snapshot2),
@@ -377,16 +382,16 @@ class CinderServersTestCase(test.ScenarioTestCase):
         scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
         scenario._delete_snapshot = mock.MagicMock()
 
-        scenario.create_nested_snapshots_and_attach_volume()
+        scenario.create_nested_snapshots_and_attach_volume(nested_level=2)
 
-        # NOTE: Two calls for random size and nested level
+        # NOTE: One call for random size
         random_call_count = mock_random.randint.call_count
-        self.assertEqual(2, random_call_count)
+        self.assertEqual(1, random_call_count)
 
         calls = scenario._create_volume.mock_calls
-        expected_calls = [mock.call(3),
-                          mock.call(3, snapshot_id=fake_snapshot.id),
-                          mock.call(3, snapshot_id=fake_snapshot.id)]
+        expected_calls = [mock.call(3)]
+        expected_calls.extend(
+            [mock.call(3, snapshot_id=fake_snapshot.id)])
         self.assertEqual(expected_calls, calls)
 
     def test_create_volume_backup(self):
