@@ -604,8 +604,8 @@ class ValidatorsTestCase(test.TestCase):
         result = validator({"args": {"a": 1, "c": 3}}, None, None)
         self.assertFalse(result.is_valid, result.msg)
 
-    @mock.patch("rally.common.objects.Endpoint")
-    def test_required_service(self, mock_endpoint):
+    @mock.patch("rally.common.objects.Credential")
+    def test_required_service(self, mock_credential):
         validator = self._unwrap_validator(validation.required_services,
                                            consts.Service.KEYSTONE,
                                            consts.Service.NOVA,
@@ -621,8 +621,8 @@ class ValidatorsTestCase(test.TestCase):
             nova_client = clients_cls.return_value.nova.return_value
             nova_client.services.list.return_value = [fake_service]
             result = validator({}, clients, {"admin": {"info": "admin"}})
-            clients_cls.assert_called_once_with(mock_endpoint.return_value)
-            mock_endpoint.assert_called_once_with(info="admin")
+            clients_cls.assert_called_once_with(mock_credential.return_value)
+            mock_credential.assert_called_once_with(info="admin")
         self.assertTrue(result.is_valid, result.msg)
 
         validator = self._unwrap_validator(validation.required_services,
@@ -661,7 +661,7 @@ class ValidatorsTestCase(test.TestCase):
                                            admin=True)
 
         # admin presented in deployment
-        fake_deployment = {"admin": "admin_endpoint", "users": []}
+        fake_deployment = {"admin": "admin_credential", "users": []}
         self.assertTrue(validator(None, None, fake_deployment).is_valid)
 
         # admin not presented in deployment
@@ -673,7 +673,7 @@ class ValidatorsTestCase(test.TestCase):
                                            users=True)
 
         # users presented in deployment
-        fake_deployment = {"admin": None, "users": ["u_endpoint"]}
+        fake_deployment = {"admin": None, "users": ["u_credential"]}
         self.assertTrue(validator({}, None, fake_deployment).is_valid)
 
         # admin and users presented in deployment
@@ -768,11 +768,11 @@ class ValidatorsTestCase(test.TestCase):
         clients.keystone.return_value = "keystone"
         clients.nova.return_value = "nova"
         mock_osclients.Clients.return_value = clients
-        mock_objects.Endpoint.return_value = "foo_endpoint"
+        mock_objects.Credential.return_value = "foo_credential"
         result = validator({}, clients, {"admin": {"foo": "bar"}})
         self.assertTrue(result.is_valid, result.msg)
-        mock_objects.Endpoint.assert_called_once_with(foo="bar")
-        mock_osclients.Clients.assert_called_once_with("foo_endpoint")
+        mock_objects.Credential.assert_called_once_with(foo="bar")
+        mock_osclients.Clients.assert_called_once_with("foo_credential")
         clients.nova.side_effect = ImportError
         result = validator({}, clients, {"admin": {"foo": "bar"}})
         self.assertFalse(result.is_valid, result.msg)
@@ -800,15 +800,15 @@ class ValidatorsTestCase(test.TestCase):
             validation.required_cinder_services,
             service_name=six.text_type("cinder-service"))
 
-        with mock.patch.object(rally.osclients.Clients, "cinder") as client:
+        with mock.patch.object(rally.osclients.Cinder, "create_client") as c:
             fake_service = mock.Mock(binary="cinder-service", state="up")
             cinder_client = mock.Mock()
             services = mock.Mock()
             services.list.return_value = [fake_service]
             cinder_client.services = services
-            client.return_value = cinder_client
+            c.return_value = cinder_client
 
-            deployment = {"admin": {"auth_url": "fake_endpoint",
+            deployment = {"admin": {"auth_url": "fake_credential",
                                     "username": "username",
                                     "password": "password"}}
             result = validator({}, None, deployment)
