@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
+import ddt
+from keystoneclient.auth import token_endpoint
 from keystoneclient import exceptions as keystone_exceptions
 import mock
 from oslo_config import cfg
@@ -26,26 +27,53 @@ from tests.unit import fakes
 from tests.unit import test
 
 
+@osclients.configure("dummy")
+class DummyClient(osclients.OSClient):
+    def create_client(self, *args, **kwargs):
+        pass
+
+
+class OSClientTestCase(test.TestCase):
+    def test_choose_service_type(self):
+        default_service_type = "default_service_type"
+
+        @osclients.configure("test_choose_service_type",
+                             default_service_type=default_service_type)
+        class FakeClient(osclients.OSClient):
+            create_client = mock.MagicMock()
+
+        fake_client = FakeClient(mock.MagicMock(), {}, {})
+        self.assertEqual(default_service_type,
+                         fake_client.choose_service_type())
+        self.assertEqual("foo",
+                         fake_client.choose_service_type("foo"))
+
+
 class CachedTestCase(test.TestCase):
 
     def test_cached(self):
-        foo_client = mock.Mock(
-            __name__="foo_client",
-            side_effect=lambda ins, *args, **kw: (args, kw))
-        ins = mock.Mock(cache={})
-        cached = osclients.cached(foo_client)
-        self.assertEqual(((), {}), cached(ins))
-        self.assertEqual({"foo_client": ((), {})}, ins.cache)
-        self.assertEqual((("foo",), {"bar": "spam"}),
-                         cached(ins, "foo", bar="spam"))
+        clients = osclients.Clients(mock.MagicMock())
+        client_name = "CachedTestCase.test_cached"
+        fake_client = osclients.configure(client_name)(
+            osclients.OSClient(clients.credential, clients.api_info,
+                               clients.cache))
+        fake_client.create_client = mock.MagicMock()
+
+        self.assertEqual({}, clients.cache)
+        fake_client()
         self.assertEqual(
-            {"foo_client": ((), {}),
-             "foo_client('foo',){'bar': 'spam'}": (("foo",),
-                                                   {"bar": "spam"})},
-            ins.cache)
-        ins.cache["foo_client('foo',){'bar': 'spam'}"] = "foo_cached"
+            {client_name: fake_client.create_client.return_value},
+            clients.cache)
+        fake_client.create_client.assert_called_once_with()
+        fake_client()
+        fake_client.create_client.assert_called_once_with()
+        fake_client("2")
         self.assertEqual(
-            "foo_cached", cached(ins, "foo", bar="spam"))
+            {client_name: fake_client.create_client.return_value,
+             "%s('2',)" % client_name: fake_client.create_client.return_value},
+            clients.cache)
+        clients.clear()
+        self.assertEqual({}, clients.cache)
 
 
 class TestCreateKeystoneClient(test.TestCase):
@@ -66,7 +94,7 @@ class TestCreateKeystoneClient(test.TestCase):
         with mock.patch.dict("sys.modules",
                              {"keystoneclient": mock_keystone,
                               "keystoneclient.v2_0": mock_keystone.v2_0}):
-            client = osclients.create_keystone_client(self.kwargs)
+            client = osclients.Keystone._create_keystone_client(self.kwargs)
             mock_discover.version_data.assert_called_once_with()
             self.assertEqual(fake_keystoneclient, client)
             mock_keystone.v2_0.client.Client.assert_called_once_with(
@@ -82,7 +110,7 @@ class TestCreateKeystoneClient(test.TestCase):
         with mock.patch.dict("sys.modules",
                              {"keystoneclient": mock_keystone,
                               "keystoneclient.v3": mock_keystone.v3}):
-            client = osclients.create_keystone_client(self.kwargs)
+            client = osclients.Keystone._create_keystone_client(self.kwargs)
             mock_discover.version_data.assert_called_once_with()
             self.assertEqual(fake_keystoneclient, client)
             mock_keystone.v3.client.Client.assert_called_once_with(
@@ -95,24 +123,27 @@ class TestCreateKeystoneClient(test.TestCase):
         mock_keystone.discover.Discover.return_value = mock_discover
         with mock.patch.dict("sys.modules", {"keystoneclient": mock_keystone}):
             self.assertRaises(exceptions.RallyException,
-                              osclients.create_keystone_client, self.kwargs)
+                              osclients.Keystone._create_keystone_client,
+                              self.kwargs)
             mock_discover.version_data.assert_called_once_with()
 
 
+@ddt.ddt
 class OSClientsTestCase(test.TestCase):
 
     def setUp(self):
         super(OSClientsTestCase, self).setUp()
-        self.endpoint = objects.Endpoint("http://auth_url", "use", "pass",
-                                         "tenant")
-        self.clients = osclients.Clients(self.endpoint)
+        self.credential = objects.Credential("http://auth_url", "use", "pass",
+                                             "tenant")
+        self.clients = osclients.Clients(self.credential, {})
 
         self.fake_keystone = fakes.FakeKeystoneClient()
         self.fake_keystone.auth_token = mock.MagicMock()
         self.service_catalog = self.fake_keystone.service_catalog
         self.service_catalog.url_for = mock.MagicMock()
 
-        keystone_patcher = mock.patch("rally.osclients.create_keystone_client")
+        keystone_patcher = mock.patch(
+            "rally.osclients.Keystone._create_keystone_client")
         self.mock_create_keystone_client = keystone_patcher.start()
         self.addCleanup(keystone_patcher.stop)
         self.mock_create_keystone_client.return_value = self.fake_keystone
@@ -129,41 +160,95 @@ class OSClientsTestCase(test.TestCase):
                               "OS_REGION_NAME": "foo_region_name"}):
             clients = osclients.Clients.create_from_env()
 
-        self.assertEqual("foo_auth_url", clients.endpoint.auth_url)
-        self.assertEqual("foo_username", clients.endpoint.username)
-        self.assertEqual("foo_password", clients.endpoint.password)
-        self.assertEqual("foo_tenant_name", clients.endpoint.tenant_name)
-        self.assertEqual("foo_region_name", clients.endpoint.region_name)
+        self.assertEqual("foo_auth_url", clients.credential.auth_url)
+        self.assertEqual("foo_username", clients.credential.username)
+        self.assertEqual("foo_password", clients.credential.password)
+        self.assertEqual("foo_tenant_name", clients.credential.tenant_name)
+        self.assertEqual("foo_region_name", clients.credential.region_name)
+
+    @mock.patch.object(DummyClient, "_get_endpoint")
+    @mock.patch("keystoneclient.session.Session")
+    def test_get_session(self, mock_session, mock_dummy_client__get_endpoint):
+        # Use DummyClient since if not the abc meta kicks in
+        osc = DummyClient(self.credential, {}, {})
+
+        with mock.patch.object(token_endpoint, "Token") as token:
+            osc._get_session()
+
+            token.assert_called_once_with(
+                mock_dummy_client__get_endpoint.return_value,
+                self.fake_keystone.auth_token
+            )
+            mock_session.assert_called_once_with(
+                auth=token.return_value, verify=False,
+                timeout=cfg.CONF.openstack_client_http_timeout)
+
+    @mock.patch.object(DummyClient, "_get_endpoint")
+    @mock.patch("keystoneclient.session.Session")
+    def test_get_session_with_endpoint(
+            self, mock_session, mock_dummy_client__get_endpoint):
+        # Use DummyClient since if not the abc meta kicks in
+        osc = DummyClient(self.credential, {}, {})
+
+        fake_endpoint = mock.Mock()
+        with mock.patch.object(token_endpoint, "Token") as token:
+            osc._get_session(endpoint=fake_endpoint)
+
+            self.assertFalse(mock_dummy_client__get_endpoint.called)
+
+            token.assert_called_once_with(
+                fake_endpoint,
+                self.fake_keystone.auth_token
+            )
+            mock_session.assert_called_once_with(
+                auth=token.return_value, verify=False,
+                timeout=cfg.CONF.openstack_client_http_timeout)
+
+    @mock.patch("keystoneclient.session.Session")
+    def test_get_session_with_auth(self, mock_session):
+        # Use DummyClient since if not the abc meta kicks in
+        osc = DummyClient(self.credential, {}, {})
+
+        fake_auth = mock.Mock()
+        osc._get_session(auth=fake_auth)
+
+        mock_session.assert_called_once_with(
+            auth=fake_auth, verify=False,
+            timeout=cfg.CONF.openstack_client_http_timeout)
 
     def test_keystone(self):
         self.assertNotIn("keystone", self.clients.cache)
         client = self.clients.keystone()
         self.assertEqual(client, self.fake_keystone)
-        endpoint = {"timeout": cfg.CONF.openstack_client_http_timeout,
-                    "insecure": False, "cacert": None}
-        kwargs = self.endpoint.to_dict()
-        kwargs.update(endpoint.items())
+        credential = {"timeout": cfg.CONF.openstack_client_http_timeout,
+                      "insecure": False, "cacert": None}
+        kwargs = self.credential.to_dict()
+        kwargs.update(credential.items())
         self.mock_create_keystone_client.assert_called_once_with(kwargs)
         self.assertEqual(self.fake_keystone, self.clients.cache["keystone"])
 
-    @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_user_not_admin(self, mock_clients_keystone):
-        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_clients_keystone.return_value.auth_ref.role_names = ["notadmin"]
+    @mock.patch("rally.osclients.Keystone.create_client")
+    def test_verified_keystone_user_not_admin(self,
+                                              mock_keystone_create_client):
+        # naming rule for mocks sucks
+        mock_keystone = mock_keystone_create_client
+        mock_keystone.return_value = fakes.FakeKeystoneClient()
+        mock_keystone.return_value.auth_ref.role_names = ["notadmin"]
         self.assertRaises(exceptions.InvalidAdminException,
                           self.clients.verified_keystone)
 
-    @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_unauthorized(self, mock_clients_keystone):
-        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_clients_keystone.side_effect = keystone_exceptions.Unauthorized
+    @mock.patch("rally.osclients.Keystone.create_client")
+    def test_verified_keystone_unauthorized(self, mock_keystone_create_client):
+        mock_keystone_create_client.return_value = fakes.FakeKeystoneClient()
+        mock_keystone_create_client.side_effect = (
+            keystone_exceptions.Unauthorized)
         self.assertRaises(exceptions.InvalidEndpointsException,
                           self.clients.verified_keystone)
 
-    @mock.patch("rally.osclients.Clients.keystone")
-    def test_verified_keystone_unreachable(self, mock_clients_keystone):
-        mock_clients_keystone.return_value = fakes.FakeKeystoneClient()
-        mock_clients_keystone.side_effect = (
+    @mock.patch("rally.osclients.Keystone.create_client")
+    def test_verified_keystone_unreachable(self, mock_keystone_create_client):
+        mock_keystone_create_client.return_value = fakes.FakeKeystoneClient()
+        mock_keystone_create_client.side_effect = (
             keystone_exceptions.AuthorizationFailure
         )
         self.assertRaises(exceptions.HostUnreachableException,
@@ -180,17 +265,17 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="compute",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             mock_nova.client.Client.assert_called_once_with(
                 "2",
                 auth_token=self.fake_keystone.auth_token,
                 http_log_debug=False,
                 timeout=cfg.CONF.openstack_client_http_timeout,
                 insecure=False, cacert=None,
-                username=self.endpoint.username,
-                api_key=self.endpoint.password,
-                project_id=self.endpoint.tenant_name,
-                auth_url=self.endpoint.auth_url)
+                username=self.credential.username,
+                api_key=self.credential.password,
+                project_id=self.credential.tenant_name,
+                auth_url=self.credential.auth_url)
             client.set_management_url.assert_called_once_with(
                 self.service_catalog.url_for.return_value)
             self.assertEqual(fake_nova, self.clients.cache["nova"])
@@ -208,17 +293,17 @@ class OSClientsTestCase(test.TestCase):
                 "token": self.fake_keystone.auth_token,
                 "endpoint_url": self.service_catalog.url_for.return_value,
                 "timeout": cfg.CONF.openstack_client_http_timeout,
-                "insecure": self.endpoint.insecure,
-                "ca_cert": self.endpoint.cacert,
-                "username": self.endpoint.username,
-                "password": self.endpoint.password,
-                "tenant_name": self.endpoint.tenant_name,
-                "auth_url": self.endpoint.auth_url
+                "insecure": self.credential.insecure,
+                "ca_cert": self.credential.cacert,
+                "username": self.credential.username,
+                "password": self.credential.password,
+                "tenant_name": self.credential.tenant_name,
+                "auth_url": self.credential.auth_url
             }
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="network",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             mock_neutron.client.Client.assert_called_once_with("2.0", **kw)
             self.assertEqual(fake_neutron, self.clients.cache["neutron"])
 
@@ -237,7 +322,7 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="image",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             mock_glance.Client.assert_called_once_with("1", **kw)
             self.assertEqual(fake_glance, self.clients.cache["glance"])
 
@@ -252,16 +337,16 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="volume",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             mock_cinder.client.Client.assert_called_once_with(
                 "1",
                 http_log_debug=False,
                 timeout=cfg.CONF.openstack_client_http_timeout,
                 insecure=False, cacert=None,
-                username=self.endpoint.username,
-                api_key=self.endpoint.password,
-                project_id=self.endpoint.tenant_name,
-                auth_url=self.endpoint.auth_url)
+                username=self.credential.username,
+                api_key=self.credential.password,
+                project_id=self.credential.tenant_name,
+                auth_url=self.credential.auth_url)
             self.assertEqual(fake_cinder.client.management_url,
                              self.service_catalog.url_for.return_value)
             self.assertEqual(fake_cinder.client.auth_token,
@@ -277,17 +362,17 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="share",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             mock_manila.client.Client.assert_called_once_with(
                 "1",
                 http_log_debug=False,
                 timeout=cfg.CONF.openstack_client_http_timeout,
                 insecure=False, cacert=None,
-                username=self.endpoint.username,
-                api_key=self.endpoint.password,
-                region_name=self.endpoint.region_name,
-                project_name=self.endpoint.tenant_name,
-                auth_url=self.endpoint.auth_url)
+                username=self.credential.username,
+                api_key=self.credential.password,
+                region_name=self.credential.region_name,
+                project_name=self.credential.tenant_name,
+                auth_url=self.credential.auth_url)
             self.assertEqual(
                 mock_manila.client.Client.return_value.client.management_url,
                 self.service_catalog.url_for.return_value)
@@ -311,21 +396,49 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="metering",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             kw = {"os_endpoint": self.service_catalog.url_for.return_value,
                   "token": self.fake_keystone.auth_token,
                   "timeout": cfg.CONF.openstack_client_http_timeout,
                   "insecure": False, "cacert": None,
-                  "username": self.endpoint.username,
-                  "password": self.endpoint.password,
-                  "tenant_name": self.endpoint.tenant_name,
-                  "auth_url": self.endpoint.auth_url
+                  "username": self.credential.username,
+                  "password": self.credential.password,
+                  "tenant_name": self.credential.tenant_name,
+                  "auth_url": self.credential.auth_url
                   }
             mock_ceilometer.client.get_client.assert_called_once_with("2",
                                                                       **kw)
             self.assertEqual(fake_ceilometer,
                              self.clients.cache["ceilometer"])
 
+    def test_monasca(self):
+        fake_monasca = fakes.FakeMonascaClient()
+        mock_monasca = mock.MagicMock()
+        mock_monasca.client.Client.return_value = fake_monasca
+        self.assertNotIn("monasca", self.clients.cache)
+        with mock.patch.dict("sys.modules",
+                             {"monascaclient": mock_monasca}):
+            client = self.clients.monasca()
+            self.assertEqual(fake_monasca, client)
+            self.service_catalog.url_for.assert_called_once_with(
+                service_type="monitoring",
+                endpoint_type=consts.EndpointType.PUBLIC,
+                region_name=self.credential.region_name)
+            os_endpoint = self.service_catalog.url_for.return_value
+            kw = {"token": self.fake_keystone.auth_token,
+                  "timeout": cfg.CONF.openstack_client_http_timeout,
+                  "insecure": False, "cacert": None,
+                  "username": self.credential.username,
+                  "password": self.credential.password,
+                  "tenant_name": self.credential.tenant_name,
+                  "auth_url": self.credential.auth_url
+                  }
+            mock_monasca.client.Client.assert_called_once_with("2_0",
+                                                               os_endpoint,
+                                                               **kw)
+            self.assertEqual(mock_monasca.client.Client.return_value,
+                             self.clients.cache["monasca"])
+
     def test_ironic(self):
         fake_ironic = fakes.FakeIronicClient()
         mock_ironic = mock.MagicMock()
@@ -338,13 +451,13 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="baremetal",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             kw = {
                 "os_auth_token": self.fake_keystone.auth_token,
                 "ironic_url": self.service_catalog.url_for.return_value,
                 "timeout": cfg.CONF.openstack_client_http_timeout,
-                "insecure": self.endpoint.insecure,
-                "cacert": self.endpoint.cacert
+                "insecure": self.credential.insecure,
+                "cacert": self.credential.cacert
             }
             mock_ironic.client.get_client.assert_called_once_with("1", **kw)
             self.assertEqual(fake_ironic, self.clients.cache["ironic"])
@@ -358,12 +471,13 @@ class OSClientsTestCase(test.TestCase):
             client = self.clients.sahara()
             self.assertEqual(fake_sahara, client)
             kw = {
-                "username": self.endpoint.username,
-                "api_key": self.endpoint.password,
-                "project_name": self.endpoint.tenant_name,
-                "auth_url": self.endpoint.auth_url
+                "service_type": "data-processing",
+                "username": self.credential.username,
+                "api_key": self.credential.password,
+                "project_name": self.credential.tenant_name,
+                "auth_url": self.credential.auth_url
             }
-            mock_sahara.client.Client.assert_called_once_with("1.1", **kw)
+            mock_sahara.client.Client.assert_called_once_with(1.1, **kw)
             self.assertEqual(fake_sahara, self.clients.cache["sahara"])
 
     def test_zaqar(self):
@@ -378,15 +492,15 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="messaging",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             fake_zaqar_url = self.service_catalog.url_for.return_value
             conf = {"auth_opts": {"backend": "keystone", "options": {
-                "os_username": self.endpoint.username,
-                "os_password": self.endpoint.password,
-                "os_project_name": self.endpoint.tenant_name,
+                "os_username": self.credential.username,
+                "os_password": self.credential.password,
+                "os_project_name": self.credential.tenant_name,
                 "os_project_id": self.fake_keystone.auth_tenant_id,
-                "os_auth_url": self.endpoint.auth_url,
-                "insecure": self.endpoint.insecure,
+                "os_auth_url": self.credential.auth_url,
+                "insecure": self.credential.insecure,
             }}}
             mock_zaqar.client.Client.assert_called_once_with(
                 url=fake_zaqar_url, version=1.1, conf=conf)
@@ -401,14 +515,14 @@ class OSClientsTestCase(test.TestCase):
             client = self.clients.trove()
             self.assertEqual(fake_trove, client)
             kw = {
-                "username": self.endpoint.username,
-                "api_key": self.endpoint.password,
-                "project_id": self.endpoint.tenant_name,
-                "auth_url": self.endpoint.auth_url,
-                "region_name": self.endpoint.region_name,
+                "username": self.credential.username,
+                "api_key": self.credential.password,
+                "project_id": self.credential.tenant_name,
+                "auth_url": self.credential.auth_url,
+                "region_name": self.credential.region_name,
                 "timeout": cfg.CONF.openstack_client_http_timeout,
-                "insecure": self.endpoint.insecure,
-                "cacert": self.endpoint.cacert
+                "insecure": self.credential.insecure,
+                "cacert": self.credential.cacert
             }
             mock_trove.client.Client.assert_called_once_with("1.0", **kw)
             self.assertEqual(fake_trove, self.clients.cache["trove"])
@@ -427,7 +541,7 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="workflowv2",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name
+                region_name=self.credential.region_name
             )
             fake_mistral_url = self.service_catalog.url_for.return_value
             mock_mistral.client.client.assert_called_once_with(
@@ -448,16 +562,16 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="object-store",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             kw = {"retries": 1,
                   "preauthurl": self.service_catalog.url_for.return_value,
                   "preauthtoken": self.fake_keystone.auth_token,
                   "insecure": False,
                   "cacert": None,
-                  "user": self.endpoint.username,
-                  "key": self.endpoint.password,
-                  "tenant_name": self.endpoint.tenant_name,
-                  "authurl": self.endpoint.auth_url
+                  "user": self.credential.username,
+                  "key": self.credential.password,
+                  "tenant_name": self.credential.tenant_name,
+                  "authurl": self.credential.auth_url
                   }
             mock_swift.client.Connection.assert_called_once_with(**kw)
             self.assertEqual(self.clients.cache["swift"], fake_swift)
@@ -478,25 +592,25 @@ class OSClientsTestCase(test.TestCase):
             self.service_catalog.url_for.assert_called_once_with(
                 service_type="ec2",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name)
+                region_name=self.credential.region_name)
             kw = {
                 "url": "http://fake.to:1/fake",
                 "aws_access_key_id": "fake_access",
                 "aws_secret_access_key": "fake_secret",
-                "is_secure": self.endpoint.insecure,
+                "is_secure": self.credential.insecure,
             }
             mock_boto.connect_ec2_endpoint.assert_called_once_with(**kw)
             self.assertEqual(fake_ec2, self.clients.cache["ec2"])
 
-    @mock.patch("rally.osclients.Clients.keystone")
-    def test_services(self, mock_clients_keystone):
+    @mock.patch("rally.osclients.Keystone.create_client")
+    def test_services(self, mock_keystone_create_client):
         available_services = {consts.ServiceType.IDENTITY: {},
                               consts.ServiceType.COMPUTE: {},
                               "unknown_service": {}}
-        mock_clients_keystone.return_value = mock.Mock(
+        mock_keystone_create_client.return_value = mock.Mock(
             service_catalog=mock.Mock(
                 get_endpoints=lambda: available_services))
-        clients = osclients.Clients(self.endpoint)
+        clients = osclients.Clients(self.credential)
 
         self.assertEqual(
             {consts.ServiceType.IDENTITY: consts.Service.KEYSTONE,
@@ -512,31 +626,75 @@ class OSClientsTestCase(test.TestCase):
             client = self.clients.murano()
             self.assertEqual(fake_murano, client)
             self.service_catalog.url_for.assert_called_once_with(
-                service_type="application_catalog",
+                service_type="application-catalog",
                 endpoint_type=consts.EndpointType.PUBLIC,
-                region_name=self.endpoint.region_name
+                region_name=self.credential.region_name
             )
             kw = {"endpoint": self.service_catalog.url_for.return_value,
                   "token": self.fake_keystone.auth_token}
             mock_murano.client.Client.assert_called_once_with("1", **kw)
             self.assertEqual(fake_murano, self.clients.cache["murano"])
 
-    @mock.patch("rally.osclients.cached")
-    def test_register(self, mock_cached):
-        client_func = mock.Mock(return_value="foo_client")
-        cached_client_func = mock.Mock(return_value="cached_foo_client")
-        mock_cached.return_value = cached_client_func
-        clients = osclients.Clients(mock.Mock())
+    @mock.patch("rally.osclients.Designate._get_session")
+    @ddt.data(
+        {},
+        {"version": "2"},
+        {"version": "1"},
+        {"version": None}
+    )
+    @ddt.unpack
+    def test_designate(self, mock_designate__get_session, version=None):
+        fake_designate = fakes.FakeDesignateClient()
+        mock_designate = mock.Mock()
+        mock_designate.client.Client.return_value = fake_designate
 
-        self.assertFalse(hasattr(clients, "foo"))
+        mock_designate__get_session.return_value = self.fake_keystone.session
 
-        func = osclients.Clients.register("foo")(client_func)
+        self.assertNotIn("designate", self.clients.cache)
+        with mock.patch.dict("sys.modules",
+                             {"designateclient": mock_designate}):
+            if version is not None:
+                client = self.clients.designate(version=version)
+            else:
+                client = self.clients.designate()
+            self.assertEqual(fake_designate, client)
+            self.service_catalog.url_for.assert_called_once_with(
+                service_type="dns",
+                endpoint_type=consts.EndpointType.PUBLIC,
+                region_name=self.credential.region_name
+            )
 
-        mock_cached.assert_called_once_with(client_func)
-        self.assertEqual("cached_foo_client", clients.foo())
-        self.assertEqual(client_func, func)
-        self.assertEqual(cached_client_func, clients.foo)
+            default = version or "1"
 
-        # Call second time with same name
-        self.assertRaises(ValueError,
-                          osclients.Clients.register("foo"), client_func)
+            # Check that we append /v<version>
+            url = self.service_catalog.url_for.return_value
+            url.__iadd__.assert_called_once_with("/v%s" % default)
+
+            mock_designate__get_session.assert_called_once_with(
+                endpoint=url.__iadd__.return_value)
+
+            mock_designate.client.Client.assert_called_once_with(
+                default, session=self.fake_keystone.session)
+
+            key = "designate"
+            if version is not None:
+                key += "%s" % {"version": version}
+            self.assertEqual(fake_designate, self.clients.cache[key])
+
+    @mock.patch("rally.osclients.Cue._get_session")
+    def test_cue(self, mock_cue__get_session):
+        fake_cue = fakes.FakeCueClient()
+        mock_cue = mock.MagicMock()
+        mock_cue.client.Client = mock.MagicMock(return_value=fake_cue)
+
+        mock_cue__get_session.return_value = self.fake_keystone.session
+
+        self.assertNotIn("cue", self.clients.cache)
+        with mock.patch.dict("sys.modules", {"cueclient": mock_cue,
+                                             "cueclient.v1": mock_cue}):
+            client = self.clients.cue()
+            self.assertEqual(fake_cue, client)
+            mock_cue.client.Client.assert_called_once_with(
+                interface=consts.EndpointType.PUBLIC,
+                session=self.fake_keystone.session)
+            self.assertEqual(fake_cue, self.clients.cache["cue"])
